import { Component, inject } from '@angular/core';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { ActivatedRoute, RouterModule } from '@angular/router';
import { Group } from '../../shared/models/group';
import GroupService from '../../shared/services/group.service';
import { MatTableModule } from '@angular/material/table';
import { GroupListComponent } from "../group-list/group-list.component";
import { Policy } from '../../shared/models/policy';
import { MatDialog } from '@angular/material/dialog';
import { GroupItemEditComponent } from '../group-item-edit/group-item-edit.component';
import { MatMenuModule } from '@angular/material/menu';
import { HermesClientService } from '../../hermes-client.service';
import { GroupChatter } from '../../shared/models/group-chatter';

@Component({
  selector: 'groups',
  imports: [
    MatButtonModule,
    MatIconModule,
    MatMenuModule,
    MatTableModule,
    RouterModule,
    GroupListComponent,
  ],
  templateUrl: './groups.component.html',
  styleUrl: './groups.component.scss'
})
export class GroupsComponent {
  private readonly _groupService = inject(GroupService);
  private readonly _client = inject(HermesClientService);
  private readonly _route = inject(ActivatedRoute);
  private readonly _dialog = inject(MatDialog);

  items: { group: Group, chatters: GroupChatter[], policies: Policy[] }[] = [];

  constructor() {
    this._route.data.subscribe(payload => {
      const groups = payload['groups'];
      const chatters = payload['chatters'];
      const policies = payload['policies'];
      const elements: { group: Group, chatters: GroupChatter[], policies: Policy[] }[] = [];

      for (let group of groups) {
        elements.push({
          group: group,
          chatters: chatters.filter((c: GroupChatter) => c.group_id == group.id),
          policies: policies.filter((p: Policy) => p.group_id == group.id),
        });
      }

      this.items = elements;
    });

    this._groupService.createGroup$?.subscribe(d => {
      if (d.error || !d.data || d.request.nounce != null && d.request.nounce.startsWith(this._client.session_id))
        return;

      let index = -1;
      for (let i = 0; i < this.items.length; i++) {
        const comp = this.compare(d.data, this.items[i].group);
        if (comp < 0) {
          index = i;
          break;
        }
      }
      this.items.splice(index >= 0 ? index : this.items.length, 0, { group: d.data, chatters: [], policies: [] });
    });

    this._groupService.updateGroup$?.subscribe(d => {
      if (d.error || !d.data || d.request.nounce != null && d.request.nounce.startsWith(this._client.session_id))
        return;

      const group = this.items.find(r => r.group.id = d.data.id)?.group;
      if (group) {
        group.id = d.data.id;
        group.name = d.data.name;
        group.priority = d.data.priority;
      }
    });

    this._groupService.deleteGroup$?.subscribe(d => {
      if (d.error || d.request.nounce != null && d.request.nounce.startsWith(this._client.session_id))
        return;

      this.items = this.items.filter(r => r.group.id != d.request.data.id);
    });
  }


  openDialog(groupName: string): void {
    const group = { id: '', user_id: '', name: groupName, priority: 0 };
    const dialogRef = this._dialog.open(GroupItemEditComponent, {
      data: { group, isSpecial: groupName.length > 0 },
    });

    const isNewGroup = group.id.length <= 0;
    dialogRef.afterClosed().subscribe((result: Group | undefined) => {
      if (!result)
        return;


      if (isNewGroup) {
        this.items.push({ group: result, chatters: [], policies: [] });
      } else {
        const same = this.items.find(i => i.group.id == group.id);
        if (same == null)
          return;

        same.group.name = result.name;
        same.group.priority = result.priority;
      }
    });
  }

  compare(a: Group, b: Group) {
    return a.name.localeCompare(b.name);
  }
}